@@ -20,12 +20,6 @@
 #define LABEL_DATA "gnome-keybinding-properties-label"
 #define MAX_ELEMENTS_BEFORE_SCROLLING 10
 
-#if defined(__powerpc__) && defined (__linux__)
-#define USE_FBLEVEL
-#else
-#undef USE_FBLEVEL
-#endif
-
 typedef enum {
   ALWAYS_VISIBLE,
   N_WORKSPACES_GT
@@ -53,10 +47,6 @@ static const KeyListEntry desktop_key_list[] =
   { "/apps/metacity/global_keybindings/run_command_screenshot", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/run_command_window_screenshot", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/run_command_terminal", ALWAYS_VISIBLE, 0 },
-#ifdef USE_FBLEVEL
-  { "/apps/gnome_settings_daemon/keybindings/brightness_down", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/brightness_up", ALWAYS_VISIBLE, 0 },
-#endif
   { NULL }
 };
 static const KeyListEntry sounds_key_list[] =
@@ -65,11 +55,11 @@ static const KeyListEntry sounds_key_list[] =
   { "/apps/gnome_settings_daemon/keybindings/volume_down", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/volume_up", ALWAYS_VISIBLE, 0 },
   /* Other ones that need keysyms bindings */
-  { "/apps/gnome_settings_daemon/keybindings/play", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/pause", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/stop", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/previous", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/next", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/play", ALWAYS_VISIBLE, 0 },        
+  { "/apps/gnome_settings_daemon/keybindings/pause", ALWAYS_VISIBLE, 0 },       
+  { "/apps/gnome_settings_daemon/keybindings/stop", ALWAYS_VISIBLE, 0 },        
+  { "/apps/gnome_settings_daemon/keybindings/previous", ALWAYS_VISIBLE, 0 },    
+  { "/apps/gnome_settings_daemon/keybindings/next", ALWAYS_VISIBLE, 0 },        
   { "/apps/gnome_settings_daemon/keybindings/eject", ALWAYS_VISIBLE, 0 },
   { NULL }
 };
@@ -651,6 +641,44 @@ accel_edited_callback (GtkCellRendererText   *cell,
       (GtkTreeModelForeachFunc) cb_check_for_uniqueness,
       &tmp_key);
 
+  /* Check for unmodified keys */
+  if (tmp_key.mask == 0 && tmp_key.keycode != 0)
+    {
+      if ((tmp_key.keyval >= GDK_a && tmp_key.keyval <= GDK_z)
+	   || (tmp_key.keyval >= GDK_A && tmp_key.keyval <= GDK_Z)
+	   || (tmp_key.keyval >= GDK_kana_fullstop && tmp_key.keyval <= GDK_kana_switch)
+	   || (tmp_key.keyval >= GDK_Arabic_comma && tmp_key.keyval <= GDK_Arabic_switch)
+	   || (tmp_key.keyval >= GDK_Serbian_dje && tmp_key.keyval <= GDK_Cyrillic_HARDSIGN)
+	   || (tmp_key.keyval >= GDK_Greek_ALPHAaccent && tmp_key.keyval <= GDK_Greek_switch)
+	   || (tmp_key.keyval >= GDK_hebrew_doublelowline && tmp_key.keyval <= GDK_Hebrew_switch)
+	   || (tmp_key.keyval >= GDK_Thai_kokai && tmp_key.keyval <= GDK_Thai_lekkao)
+	   || (tmp_key.keyval >= GDK_Hangul && tmp_key.keyval <= GDK_Hangul_J_YeorinHieuh)) {
+        GtkWidget *dialog;
+	char *name;
+
+	name = egg_virtual_accelerator_name (keyval, keycode, mask);
+
+	dialog =
+	  gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
+			  	  GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+				  GTK_MESSAGE_WARNING,
+				  GTK_BUTTONS_CANCEL,
+				  _("The shortcut \"%s\" cannot be used because it will become unusable to type using this key.\n"
+				  "Please try with a key such as Control, Alt or Shift at the same time.\n"),
+				  name);
+
+	g_free (name);
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	/* set it back to its previous value. */
+	egg_cell_renderer_keys_set_accelerator
+	  (EGG_CELL_RENDERER_KEYS (cell),
+	   key_entry->keyval, key_entry->keycode, key_entry->mask);
+	return;
+      }
+    }
+
   /* flag to see if the new accelerator was in use by something */
   if (!tmp_key.editable)
     {
