@@ -154,7 +154,8 @@ def_group_in_gconf_changed (GConfClient * client,
 }
 
 static void
-add_variant_to_available_layouts_tree (const XklConfigItemPtr configItem, 
+add_variant_to_available_layouts_tree (XklConfigRegistry * configRegistry,
+                                       const XklConfigItem * configItem, 
                                        GladeXML * chooserDialog)
 {
   GtkWidget *layoutsTree = CWID ("xkb_layouts_available");
@@ -173,7 +174,8 @@ add_variant_to_available_layouts_tree (const XklConfigItemPtr configItem,
 }
 
 static void
-add_layout_to_available_layouts_tree (const XklConfigItemPtr configItem, 
+add_layout_to_available_layouts_tree (XklConfigRegistry * configRegistry,
+                                      const XklConfigItem * configItem, 
                                       GladeXML * chooserDialog)
 {
   GtkWidget *layoutsTree = CWID ("xkb_layouts_available");
@@ -189,7 +191,7 @@ add_layout_to_available_layouts_tree (const XklConfigItemPtr configItem,
 
   current1stLevelId = configItem->name;
 
-  XklConfigEnumLayoutVariants (configItem->name,
+  xkl_config_registry_foreach_layout_variant (configRegistry, configItem->name,
 			       (ConfigItemProcessFunc)add_variant_to_available_layouts_tree, 
                                chooserDialog);
 }
@@ -300,7 +302,7 @@ xkb_layouts_prepare_selected_tree (GladeXML * dialog, GConfChangeSet * changeset
   g_signal_connect_swapped (G_OBJECT (selection), "changed",
 			    G_CALLBACK
 			    (xkb_layouts_enable_disable_buttons), dialog);
-  maxSelectedLayouts = XklGetMaxNumGroups();
+  maxSelectedLayouts = xkl_engine_get_max_num_groups (engine);
 
   gconf_client_notify_add (xkbGConfClient,
                            GSWITCHIT_CONFIG_KEY_DEFAULT_GROUP,
@@ -341,7 +343,7 @@ xkb_layouts_fill_selected_tree (GladeXML * dialog)
       char *v1, *utfVisible;
       const char *visible = (char *) curLayout->data;
       gtk_list_store_append (listStore, &iter);
-      if (GSwitchItKbdConfigGetDescriptions (visible, &sl, &l, &sv, &v))
+      if (GSwitchItKbdConfigGetDescriptions (configRegistry, visible, &sl, &l, &sv, &v))
 	visible = GSwitchItKbdConfigFormatFullLayout (l, v);
       v1 = g_strdup (visible);
       utfVisible = g_locale_to_utf8 (g_strstrip (v1), -1, NULL, NULL, NULL);
@@ -417,7 +419,7 @@ xkb_layouts_fill_available_tree (GladeXML * chooserDialog)
 			   GTK_TREE_MODEL (treeStore));
   gtk_tree_view_append_column (GTK_TREE_VIEW (treeView), column);
 
-  XklConfigEnumLayouts ((ConfigItemProcessFunc)
+  xkl_config_registry_foreach_layout (configRegistry, (ConfigItemProcessFunc)
 			add_layout_to_available_layouts_tree, chooserDialog);
 
   sort_tree_content (treeView);
@@ -519,6 +521,8 @@ xkb_layout_chooser_response(GtkDialog *dialog,
                             gint response,
                             GladeXML *chooserDialog)
 {
+  GdkRectangle rect;
+
   if (response == GTK_RESPONSE_OK)
     {
       GtkTreeSelection *selection =
@@ -556,9 +560,10 @@ xkb_layout_chooser_response(GtkDialog *dialog,
               if (!anySwitcher)
                 {
                   XklConfigItem ci;
-                  g_snprintf( ci.name, XKL_MAX_CI_NAME_LENGTH, DEFAULT_GROUP_SWITCH );           
-                  if (XklConfigFindOption( GROUP_SWITCHERS_GROUP,
-                                           &ci ))
+                  g_snprintf(ci.name, XKL_MAX_CI_NAME_LENGTH, DEFAULT_GROUP_SWITCH );           
+                  if (xkl_config_registry_find_option (configRegistry,
+                                                       GROUP_SWITCHERS_GROUP,
+                                                       &ci))
                     {
                       const gchar* id = GSwitchItKbdConfigMergeItems (GROUP_SWITCHERS_GROUP, DEFAULT_GROUP_SWITCH);
                       optionsList = g_slist_append (optionsList, g_strdup (id));
@@ -570,6 +575,10 @@ xkb_layout_chooser_response(GtkDialog *dialog,
           clear_xkb_elements_list (layoutsList);
         }
     }
+
+  gtk_window_get_position (GTK_WINDOW (dialog), &rect.x, &rect.y);
+  gtk_window_get_size (GTK_WINDOW (dialog), &rect.width, &rect.height);
+  GSwitchItPreviewSave (&rect);
 }
 
 static void
@@ -595,6 +604,7 @@ xkb_layout_choose (GladeXML * dialog)
   GladeXML* chooserDialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/gnome-keyboard-properties.glade", "xkb_layout_chooser", NULL);
   GtkWidget* chooser = CWID ( "xkb_layout_chooser");
   GtkWidget* kbdraw = NULL;
+  GtkWidget* toplevel = NULL;
 
   gtk_window_set_transient_for (GTK_WINDOW (chooser), GTK_WINDOW (WID ("keyboard_dialog")));
 
@@ -602,7 +612,7 @@ xkb_layout_choose (GladeXML * dialog)
   xkb_layout_chooser_selection_changed (chooserDialog);
   
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
-  if (!strcmp (XklGetBackendName(), "XKB"))
+  if (!strcmp (xkl_engine_get_backend_name (engine), "XKB"))
   {
     kbdraw = xkb_layout_preview_create_widget (chooserDialog);
     g_object_set_data (G_OBJECT (chooser), "kbdraw", kbdraw);
@@ -617,6 +627,18 @@ xkb_layout_choose (GladeXML * dialog)
   g_signal_connect (G_OBJECT (chooser),
 		    "response", G_CALLBACK (xkb_layout_chooser_response), chooserDialog);
 
+  toplevel = gtk_widget_get_toplevel (chooser);
+  if (GTK_WIDGET_TOPLEVEL (toplevel))
+  {
+    GdkRectangle *rect = GSwitchItPreviewLoad ();
+    if (rect != NULL)
+    {
+      gtk_window_move (GTK_WINDOW (toplevel), rect->x, rect->y);
+      gtk_window_resize (GTK_WINDOW (toplevel), rect->width, rect->height);
+      g_free (rect);
+    }
+  }
+
   gtk_dialog_run (GTK_DIALOG (chooser));
   gtk_widget_destroy (chooser);
 }
