@@ -10,6 +10,7 @@
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <glib.h>
 
 #include "gnome-theme-info.h"
 #include "capplet-util.h"
@@ -139,19 +140,30 @@ static gboolean
 transfer_done_targz_idle_cb (gpointer data)
 {
 	int status;
-	gchar *command, *filename;
+	gchar *command, *filename, *gzip, *tar;
 	theme_properties *theme_props = data;
 		
+	if (!(gzip = g_find_program_in_path("gzip"))) {
+		return FALSE;
+	}
+	if (!(tar = g_find_program_in_path("tar"))) {
+		g_free(gzip);
+		return FALSE;
+	}
 	/* this should be something more clever and nonblocking */
 	filename = g_shell_quote(theme_props->filename);
-	command = g_strdup_printf ("sh -c 'cd \"%s\"; /bin/gzip -d -c < \"%s\" | /bin/tar xf - '",
-				    theme_props->target_tmp_dir, filename);
+	command = g_strdup_printf ("sh -c 'cd \"%s\"; %s -d -c < \"%s\" | %s xf - '",
+				    theme_props->target_tmp_dir, gzip, filename, tar);
 	g_free(filename);
 	if (g_spawn_command_line_sync (command, NULL, NULL, &status, NULL) && status == 0) {
 		g_free (command);
+		g_free(gzip);
+		g_free(tar);
 		return TRUE;
 	} else {	
 		g_free (command);
+		g_free(gzip);
+		g_free(tar);
 		return FALSE;
 	}
 }
@@ -171,18 +183,29 @@ static gboolean
 transfer_done_tarbz2_idle_cb (gpointer data)
 {
 	int status;
-	gchar *command, *filename;
+	gchar *command, *filename, *bzip2, *tar;
 	theme_properties *theme_props = data;
 	
+	if (!(bzip2 = g_find_program_in_path("bzip2"))) {
+		return FALSE;
+	}
+	if (!(tar = g_find_program_in_path("tar"))) {
+		g_free(bzip2);
+		return FALSE;
+	}
 	filename = g_shell_quote(theme_props->filename);
 	/* this should be something more clever and nonblocking */
-	command = g_strdup_printf ("sh -c 'cd \"%s\"; /usr/bin/bzip2 -d -c < \"%s\" | /bin/tar xf - '",
-				   theme_props->target_tmp_dir, filename);
+	command = g_strdup_printf ("sh -c 'cd \"%s\"; %s -d -c < \"%s\" | %s xf - '",
+				   theme_props->target_tmp_dir, bzip2, filename, tar);
 	g_free (filename);
 	if (g_spawn_command_line_sync (command, NULL, NULL, &status, NULL) && status == 0) {
+		g_free(bzip2);
+		g_free(tar);
 		g_free (command);
 		return TRUE;
 	} else {
+		g_free(bzip2);
+		g_free(tar);
 		g_free (command);
 		return FALSE;
 	}
@@ -193,7 +216,7 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 {
 	GtkWidget *dialog;
 	int len = strlen (path);
-	gchar *command,**dir, *first_line, *filename;
+	gchar *command,**dir, *first_line, *filename, *gzip, *bzip2, *tar;
 	int status,theme_type;
 	theme_properties *theme_props;
 	GnomeVFSURI *theme_source_dir, *theme_dest_dir;
@@ -207,16 +230,20 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 							g_random_int());
 		
 	
-	if (path && len > 7 && ( (!strcmp (path + len - 7, ".tar.gz")) || (!strcmp (path + len - 4, ".tgz")) )) {
-		filename = g_shell_quote (path);
-		command = g_strdup_printf ("sh -c '/bin/gzip -d -c < \"%s\" | /bin/tar ft -  | head -n 1'",
-					    filename);
-		theme_props->filetype=TARGZ;
-		g_free (filename);
-	} else if (path && len > 8 && !strcmp (path + len - 8, ".tar.bz2")) {
-		filename = g_shell_quote (path);
-		command = g_strdup_printf ("sh -c '/usr/bin/bzip2 -d -c < \"%s\" | /bin/tar ft - | head -n 1'",
-					    filename);
+ 	gzip = g_find_program_in_path("gzip");
+ 	bzip2 = g_find_program_in_path("bzip2");
+ 	tar = g_find_program_in_path("tar");
+  	
+ 	if (tar && gzip && path && len > 7 && ( (!strcmp (path + len - 7, ".tar.gz")) || (!strcmp (path + len - 4, ".tgz")) )) {
+  		filename = g_shell_quote (path);
+ 		command = g_strdup_printf ("sh -c '%s -d -c < \"%s\" | %s ft -  | head -1'",
+ 					    gzip, filename, tar);
+  		theme_props->filetype=TARGZ;
+  		g_free (filename);
+ 	} else if (tar && bzip2 && path && len > 8 && !strcmp (path + len - 8, ".tar.bz2")) {
+  		filename = g_shell_quote (path);
+ 		command = g_strdup_printf ("sh -c '%s -d -c < \"%s\" | %s ft - | head -1'",
+ 					    bzip2, filename, tar);
 		theme_props->filetype=TARBZ;
 		g_free (filename);
 	} else {
@@ -230,6 +257,9 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 		gnome_vfs_unlink(path);		
 		g_free (theme_props->target_tmp_dir);
 		g_free (theme_props);
+		g_free(gzip);
+		g_free(bzip2);
+		g_free(tar);
 		return;	
 	}
 	
@@ -245,6 +275,9 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 		g_free (command);
 		g_free (theme_props->target_tmp_dir);
 		g_free (theme_props);
+		g_free(gzip);
+		g_free(bzip2);
+		g_free(tar);
 		return;	
 	}
 	
@@ -252,7 +285,7 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 	theme_props->filename=g_strdup(path);
 	
 	if (theme_props->filetype == TARBZ ) {
-		if (!g_file_test ("/usr/bin/bzip2", G_FILE_TEST_EXISTS)) {
+		if (!bzip2) {
 			dialog = gtk_message_dialog_new (NULL,
 						GTK_DIALOG_MODAL,
 						GTK_MESSAGE_ERROR,
@@ -265,6 +298,9 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 			g_free (theme_props->target_tmp_dir);
 			g_free (theme_props->filename);
 			g_free (theme_props);
+			g_free(gzip);
+			g_free(bzip2);
+			g_free(tar);
 			return;	
 		}
 		
@@ -281,12 +317,15 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 			g_free (theme_props->filename);
 			g_free (theme_props);
 			g_free (command);
+ 			g_free(gzip);
+ 			g_free(bzip2);
+ 			g_free(tar);
 			return;	
 		}
 	}
 	
 	if (theme_props->filetype == TARGZ ) {
-		if (!g_file_test ("/bin/gzip", G_FILE_TEST_EXISTS)) {
+		if (!gzip) {
 			dialog = gtk_message_dialog_new (NULL,
 						GTK_DIALOG_MODAL,
 						GTK_MESSAGE_ERROR,
@@ -299,6 +338,9 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 			g_free (theme_props->target_tmp_dir);
 			g_free (theme_props->filename);
 			g_free (theme_props);
+			g_free(gzip);
+			g_free(bzip2);
+			g_free(tar);
 			return;	
 		}
 		if (!transfer_done_targz_idle_cb(theme_props)) {
@@ -314,6 +356,9 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 			g_free (theme_props->filename);
 			g_free (theme_props);
 			g_free (command);
+			g_free(gzip);
+			g_free(bzip2);
+			g_free(tar);
 			return;	
 		}	
 	}
@@ -351,6 +396,9 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 			g_free (theme_props->theme_tmp_dir);
 			g_free (theme_props);
 			g_free (command);
+			g_free(gzip);
+			g_free(bzip2);
+			g_free(tar);
 			return;	
 		} else {
 			dialog = gtk_message_dialog_new (NULL,
@@ -365,6 +413,9 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
                         g_free (theme_props->theme_tmp_dir);
                         g_free (theme_props);
 			g_free (command);
+			g_free(gzip);
+			g_free(bzip2);
+			g_free(tar);
 			return;	
 		}
 		/* Move the Dir to the target dir */
@@ -391,6 +442,9 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 			g_free (theme_props->user_message);
                         g_free (theme_props);
 			g_free (command);
+			g_free(gzip);
+			g_free(bzip2);
+			g_free(tar);
 			return;	
 		} else {
 			dialog = gtk_message_dialog_new (NULL,
@@ -408,6 +462,9 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 			g_free (theme_props->user_message);
                         g_free (theme_props);
 			g_free (command);
+			g_free(gzip);
+			g_free(bzip2);
+			g_free(tar);
 			return;	
 		}
 		
@@ -416,31 +473,23 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 	g_free (theme_props->target_tmp_dir);
 	g_free (theme_props->filename);
 	g_free (theme_props);
+	g_free(gzip);
+	g_free(bzip2);
+	g_free(tar);
 }
 
-static void
-install_dialog_response (GtkWidget *widget, int response_id, gpointer data)
+void
+gnome_theme_install_from_uri (gchar * theme_filename)
 {
-	GladeXML *dialog = data;
 	GtkWidget *dlg;
-	gchar *filename, *path, *base, *scheme;
+	gchar *filename, *path, *base;
 	GList *src, *target;
 	GnomeVFSURI *src_uri;
-	const gchar *raw;
-	gboolean icon_theme;
+	gboolean icon_theme = FALSE;
 	gchar *temppath;
 
-	if (response_id == GTK_RESPONSE_HELP) {
-		capplet_help (GTK_WINDOW (widget),
-			"user-guide.xml",
-			"goscustdesk-12");
-		return;
-	}
 
-	if (response_id == 0) {
-		icon_theme = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "icon_theme"));
-		raw = gtk_entry_get_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (WID ("install_theme_picker")))));
-		if (raw == NULL || strlen (raw) <= 0)	{
+		if (theme_filename == NULL || strlen (theme_filename) <= 0)	{
 			GtkWidget *dialog;
 
 			dialog = gtk_message_dialog_new (NULL,
@@ -453,11 +502,7 @@ install_dialog_response (GtkWidget *widget, int response_id, gpointer data)
 			return;
 		}
 
-		if ((scheme = gnome_vfs_get_uri_scheme (raw)) == NULL && *raw != '/')
-			filename = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (WID ("install_theme_picker")), TRUE);
-		else
-			filename = g_strdup (raw);
-		g_free (scheme);
+		filename = g_strdup (theme_filename);
 		
 		if (filename == NULL)	{
 			GtkWidget *dialog;
@@ -566,15 +611,26 @@ install_dialog_response (GtkWidget *widget, int response_id, gpointer data)
 		g_signal_connect (G_OBJECT (dlg), "done",
 				  G_CALLBACK (transfer_done_cb), path);
 		gtk_widget_show (dlg);
-	}
 }
 
 void
-gnome_theme_installer_run (GtkWidget *parent, gchar *filename)
+gnome_theme_installer_run_cb (GtkWidget *button,
+				   GtkWindow *parent_window)
+{
+  gnome_theme_installer_run (parent_window, NULL);
+}
+
+
+void
+gnome_theme_installer_run (GtkWindow *parent, gchar *filename)
 {
 	static gboolean running_theme_install = FALSE;
-	GladeXML *dialog;
-	GtkWidget *widget;
+	GtkWidget *dialog;
+	static char old_folder[1024] = "";
+	gchar *filename_selected, *folder;
+
+	if (filename == NULL)
+		filename = old_folder;
 
 	if (running_theme_install)
 		return;
@@ -594,22 +650,26 @@ gnome_theme_installer_run (GtkWidget *parent, gchar *filename)
 		return;
 	}
 
-	dialog = glade_xml_new (GLADEDIR "/theme-install.glade", NULL, NULL);
-	widget = WID ("install_dialog");
+	dialog = gtk_file_chooser_dialog_new ("Select Theme", parent, GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 
-	g_signal_connect (G_OBJECT (widget), "response", G_CALLBACK (install_dialog_response), dialog);
-	gtk_window_set_transient_for (GTK_WINDOW (widget), GTK_WINDOW (parent));
-	gtk_window_set_position (GTK_WINDOW (widget), GTK_WIN_POS_CENTER_ON_PARENT);
-	if (filename)
-		gnome_file_entry_set_filename (GNOME_FILE_ENTRY (WID ("install_theme_picker")), filename);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), old_folder);
 
-	while (gtk_dialog_run (GTK_DIALOG (widget)) == GTK_RESPONSE_HELP)
-		;
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+	{
+		filename_selected = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+		gnome_theme_install_from_uri (filename_selected);
+		g_free (filename_selected);
+	}
+
+
+	folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog));
+	g_strlcpy (old_folder, folder, 255);
+	g_free (folder);
 
 	gnome_theme_details_reread_themes_from_disk();
 
-	gtk_widget_destroy (widget);
-	g_object_unref (G_OBJECT (dialog));
+
+	gtk_widget_destroy (dialog);
 
 	running_theme_install = FALSE;
 }
